// ===========================================================================
//	LAGAIconDropButton.h
// ===========================================================================
//	Apple Grayscale Appearance compliant (almost) drop icon button
//	Copyright  1996 Chrisoft (Christophe ANDRES)  All rights reserved.
//
//	You may use this source code in any application (commercial, shareware, freeware,
//	postcardware, etc), but not remove this notice (no need to acknowledge the use of
//	this class in the about box)
//	You may not sell this source code in any form. This source code may be placed on 
//	publicly accessable archive sites and source code disks. It may not be placed on 
//	profit archive sites and source code disks without the permission of the author, 
//	Christophe ANDRES.
//	
//	    This source code is distributed in the hope that it will be useful,
//	    but WITHOUT ANY WARRANTY; without even the implied warranty of
//	    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
//
//	If you make any change or improvement on this class, please send the improved/changed
//	version to : chrisoft@calva.net or Christophe ANDRES
//                                     20, rue Prosper Mrime
//                                     67100 STRASBOURG
//                                     FRANCE
//
// ===========================================================================
//	LAGAIconDropButton.cp	<- double-click + Command-D to see class implementation
//
//	LAGAIconDropButton is a subclass of LAGAIconButton, and enables the button to accept
//		dropped items. To use it, you must subclass it and override ItemIsAcceptable and
//		ReceiveDragItem (defined in LDragAndDrop)
//
//		see the comments in LAGAIconButton.h
//
//		Version : 1.2
//
//		Change History (most recent first, date in US form : mm/dd/yy):
//
//						06/30/96	ca		Public release of version 1.2
//						06/04/96	ca		Added static RegisterClass for easier class registration
//														Added guard macros in header files
//														Increased version to 1.2
//						05/13/96	ca		Increased version to 1.1
//														Added copy constructor
//														Added "on the fly" constructor
//														Removed an obsolete and anyway unimplemented constructor
//														Added change history
//						04/22/96	ca		class made available by Christophe ANDRES <chrisoft@calva.net>
//														(version 1.0)
//
//		To Do:
//


#ifndef _H_LAGAIconDropButton
#define _H_LAGAIconDropButton
#pragma once

#include "LAGAIconButton.h"
#include <LDragAndDrop.h>

class	LAGAIconDropButton : public LAGAIconButton, public LDragAndDrop
{
	public		:
		enum { class_ID = '3DDp' };
		static void RegisterClass ();																																					//	<06/04/96	ca>
		static LAGAIconDropButton* CreateAGAIconDropButtonStream (LStream *inStream);
	
		LAGAIconDropButton ();
		
		LAGAIconDropButton (LStream *inStream);
		
		LAGAIconDropButton (const LAGAIconDropButton &inOriginal);																						//	<05/13/96 ca>
		
		LAGAIconDropButton (const SPaneInfo &inPaneInfo, MessageT inClickedMessage, OSType inIconType,				//	<05/13/96 ca>
												ResIDT inIconID, Boolean inRadioButtonBehavior);

		virtual ~LAGAIconDropButton ();
		
	protected	:
		virtual void HiliteDropArea (DragReference inDragRef);
		
		virtual void UnhiliteDropArea (DragReference inDragRef);
		
		virtual Boolean ItemIsAcceptable (DragReference inDragRef, ItemReference inItemRef);
};

#endif
