// ===========================================================================
//	LAGASquareHSlider.cp
// ===========================================================================
//	Apple Grayscale Appearance compliant vertical slider control with rectangular indicator
//	Copyright  1996 Chrisoft (Christophe ANDRES)  All rights reserved.
//
//	You may use this source code in any application (commercial, shareware, freeware,
//	postcardware, etc), but not remove this notice (no need to acknowledge the use of
//	this class in the about box)
//	You may not sell this source code in any form. This source code may be placed on 
//	publicly accessable archive sites and source code disks. It may not be placed on 
//	profit archive sites and source code disks without the permission of the author, 
//	Christophe ANDRES.
//	
//	    This source code is distributed in the hope that it will be useful,
//	    but WITHOUT ANY WARRANTY; without even the implied warranty of
//	    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
//
//	If you make any change or improvement on this class, please send the improved/changed
//	version to : chrisoft@calva.net or Christophe ANDRES
//                                     20, rue Prosper Mrime
//                                     67100 STRASBOURG
//                                     FRANCE
//
// ===========================================================================
//	LAGASquareHSlider.h			<- double-click + Command-D to see class declaration
//
//	LAGASquareHSlider is my implementation of the Apple Grayscale Appearance for System 7.5
//		horizontal square slider control
//
//		This class requires LAGASliderBase.cp to be present in your project
//		This class requires AGAColors.cp to be present in your project
//
//		Version : 1.2
//
//		Change History (most recent first, date in US form : mm/dd/yy):
//
//						06/30/96	ca		Public release of version 1.2
//						06/27/96	ca		Changed checks for disabled state (check on triState_On instead of triState_Off)
//															in order that triState_Latent state is drawn as disabled
//						06/04/96	ca		Added RegisterClass method to ease registry
//														Increased version to 1.2
//						05/16/96	ca		class made available by Christophe ANDRES <chrisoft@calva.net>
//														(version 1.1)
//
//		To Do:
//

#include "LAGASquareHSlider.h"
#include "AGAColors.h"
#include <UDrawingState.h>
#include <PP_Types.h>

//	begin	<06/04/96	ca>
void LAGASquareHSlider::RegisterClass ()

{
	URegistrar::RegisterClass(LAGASquareHSlider::class_ID, (ClassCreatorFunc)LAGASquareHSlider::CreateAGASquareHSliderStream);
}
//	begin	<06/04/96	ca>

LAGASquareHSlider* LAGASquareHSlider::CreateAGASquareHSliderStream (LStream *inStream)

{
	return(new LAGASquareHSlider(inStream));
}

//-------Constructors-------------------------------------------------------------------------------------------------

LAGASquareHSlider::LAGASquareHSlider ()

{
	mTrackOriginTinkering = 1;
	mVerticalSlider = false;
}

LAGASquareHSlider::LAGASquareHSlider (LStream	*inStream) : LAGASliderBase(inStream)

{
	mTrackOriginTinkering = 1;
}

LAGASquareHSlider::LAGASquareHSlider (const LAGASquareHSlider &inOriginal) : LAGASliderBase(inOriginal)

{
	mTrackOriginTinkering = 1;
	mVerticalSlider = false;
}

LAGASquareHSlider::LAGASquareHSlider (const SPaneInfo	&inPaneInfo, MessageT inValueMessage, Boolean inBottomRightPointer,
																			Int32 inInitialValue, Int32 inMinValue, Int32 inMaxValue)
																		: LAGASliderBase(inPaneInfo, inValueMessage, false, inBottomRightPointer, inInitialValue,
																												inMinValue, inMaxValue)
{
	mTrackOriginTinkering = 1;
}

//-------Drawers----------------------------------------------------------------------------------------------------


void LAGASquareHSlider::DrawIndicator (Boolean inPushed)

{
	StColorPenState theState;
	Boolean hasColor = ::PaneInColor(this);
	Boolean disabled = (mEnabled != triState_On);																							//	<06/27/96	ca>
	Rect frame;

	theState.Normalize();
	CalcLocalFrameRect(frame);
	
	short position = GetIndicatorPosition();
	
	::SetRect(&frame, position - 6, frame.top, position + 7, frame.top + 16);
	if (hasColor)
		::RGBBackColor(&gAGAColorArray[2]);
	EraseRect(&frame);
	
	if (hasColor && !disabled)
		{
			Rect r = frame;
			::InsetRect(&r, 1, 1);
			::RGBBackColor((inPushed ? &gAGAColorArray[8] : &gAGAColorArray[5]));
			::EraseRect(&r);
		}
	if (disabled)
		if (hasColor)
			::RGBForeColor(&gAGAColorArray[8]);
		else
			PenPat(&qd.gray);
	::MoveTo(position - 5, frame.top);
	::Line(10, 0);
	::Move(1, 1);		::Line(0, 13);
	::Move(-1, 1);	::Line(-10, 0);
	::Move(-1, -1);	::Line(0, -13);
	if (hasColor)
		{
			::RGBForeColor((disabled ? &gAGAColorArray[1] : (inPushed ? &gAGAColorArray[5] : &gAGAColorArray[3])));
			::Move(1, 1);		::Line(0, 11);
			::Move(1, -12);	::Line(8, 0);
			::RGBForeColor((disabled ? &gAGAColorArray[4] : (inPushed ? &gAGAColorArray[10] : &gAGAColorArray[8])));
			::Move(1, 1);		::Line(0, 12);
			::Line(-9, 0);
			::RGBForeColor((disabled ? &gAGAColorArray[W] : (inPushed ? &gAGAColorArray[3] : &gAGAColorArray[1])));
			::Move(-1, -13);	::Line(0, 0);
		}
	if (!disabled)
		{
			if (hasColor)
				{
					::Move(2, 3);		::Line(0, 0);
					::Move(2, 0);		::Line(0, 0);
					::Move(2, 0);		::Line(0, 0);
					::RGBForeColor((inPushed ? &gAGAColorArray[12] : &gAGAColorArray[10]));
				}
			else
				::Move(7, 3);
			::Move(1, 1);		::Line(0, 6);
			::Move(-2, 0);	::Line(0, -6);
			::Move(-2, 0);	::Line(0, 6);
			if (hasColor)
				{
					::RGBForeColor((inPushed ? &gAGAColorArray[5] : &gAGAColorArray[3]));
					::Move(-1, -1);	::Line(0, -5);
					::Move(2, 0);		::Line(0, 5);
					::Move(2, 0);		::Line(0, -5);
				}
		}
}

Int16 LAGASquareHSlider::FindHotSpot (Point inPoint)

{
	Rect frame;
	
	//	The only hot spot we know of in a slider is the pointer
	GetIndicatorRect(frame);
										
	return(::PtInRect(inPoint, &frame) ? 1 : 0);
}

Boolean LAGASquareHSlider::PointInHotSpot (Point inPoint, Int16 inHotSpot)

{
	Rect frame;
	
	//	The only hot spot we know of in a slider is the pointer
	GetIndicatorRect(frame);
									
	return(::PtInRect(inPoint, &frame));
}

short LAGASquareHSlider::GetIndicatorPosition ()

{
	Rect frame;

	CalcLocalFrameRect(frame);
	double size = (frame.right - frame.left) - 20;
	double division = size / (double)(mMaxValue - mMinValue);
	
	return(frame.left + 10 + ((mValue - mMinValue) * division));
}

void LAGASquareHSlider::GetIndicatorRect (Rect &outRect)

{
	Rect frame;
	
	CalcLocalFrameRect(frame);
	short position = GetIndicatorPosition();
	
	::SetRect(&outRect, position - 6, frame.top, position + 7, frame.top + 16);
}

void LAGASquareHSlider::InitializeGhost ()

{
	Boolean hasColor = ::PaneInColor(this);
	Rect r;

	LAGASliderBase::InitializeGhost();
	
	mGhostPointer->BeginDrawing();
	
	if (hasColor)
		::RGBBackColor(&gAGAColorArray[2]);
	::SetRect(&r, 0, 0, 13, 16);
	::EraseRect(&r);
	
	if (hasColor)
		::RGBForeColor(&gAGAColorArray[7]);
	::MoveTo(1, 0);	::Line(10, 0);
	::Move(1, 1);		::Line(0, 13);
	::Move(-1, 1);	::Line(-10, 0);
	::Move(-1, -1);	::Line(0, -13);
	if (hasColor)
		{
			::RGBForeColor(&gAGAColorArray[11]);
			::Move(0, 4);		::Line(0, 0);
			::Move(0, 4);		::Line(0, 0);
			::Move(12, 0);	::Line(0, 0);
			::Move(0, -4);	::Line(0, 0);
			::RGBForeColor(&gAGAColorArray[7]);
			::Move(-1, 0);	::Line(0, 0);
			::Move(0, 4);		::Line(0, 0);
			::RGBForeColor(&gAGAColorArray[5]);
			::Move(-1, 0);	::Line(-1, 0);
			::Move(1, -4);	::Line(-1, 0);
			::Move(-1, 1);	::Line(0, 2);
			::Move(0, 2);		::Line(0, 1);
			::Move(-2, 0);	::Line(0, -1);
			::Move(0, -2);	::Line(0, -2);
			::Move(-2, 0);	::Line(0, 2);
			::Move(0, 2);		::Line(0, 1);
			::Move(-2, -2);	::Line(0, 0);
			::Move(0, -4);	::Line(0, 0);
			::RGBForeColor(&gAGAColorArray[9]);
			::Move(2, 0);		::Line(0, 0);
			::Move(2, 0);		::Line(0, 0);
			::Move(2, 0);		::Line(0, 0);
			::Move(0, 4);		::Line(0, 0);
			::Move(-2, 0);	::Line(0, 0);
			::Move(-2, 0);	::Line(0, 0);
			::ForeColor(whiteColor);
			::Move(-1, 1);	::Line(0, 0);
			::Move(0, -2);	::Line(0, -2);
			::Move(0, -2);	::Line(0, 0);
			::Move(2, 0);		::Line(0, 0);
			::Move(0, 2);		::Line(0, 2);
			::Move(0, 2);		::Line(0, 0);
			::Move(2, 0);		::Line(0, 0);
			::Move(0, -2);	::Line(0, -2);
			::Move(0, -2);	::Line(0, 0);
			::Move(3, -3);	::Line(-8, 0);
			::Move(-1, 1);	::Line(0, 2);
			::Move(0, 2);		::Line(0, 2);
			::Move(0, 2);		::Line(0, 2);
		}
	mGhostPointer->EndDrawing();

	//	Define a region to draw the pointer nicely if the ghost passes on its top
	mGhostMask = ::NewRgn();
	::OpenRgn();
	::MoveTo(1, 0);	::Line(10, 0);
	::Line(2, 1);		::Line(0, 13);
	::Line(-2, 2);	::Line(-10, 0);
	::Line(-1, -1);	::Line(0, -14);
	::Line(1, -1);
	::CloseRgn(mGhostMask);
}

void LAGASquareHSlider::DrawGhost (Point inPoint)

{
	Rect frame;
	short position;

	if (inPoint.h != mLastPosition.h)
		{
			Rect indicRect, overStep;
			
			CalcLocalFrameRect(frame);
			double size = (frame.right - frame.left) - 20;
			double division = size / (double)(mMaxValue - mMinValue);
			
			short MaxPt = frame.left + 10 + ((mMaxValue - mMinValue) * division);
			short MinPt = frame.left + 10;
			if (inPoint.h > MaxPt)
				position = MaxPt;
			else
				if (inPoint.h < MinPt)
					position = MinPt;
				else
					position = inPoint.h;
			
			frame.bottom = frame.top + 16;
			frame.left = position - 6;
			frame.right = frame.left + 13;
			
			if (!::EmptyRect(&mLastGhost))
				{
					indicRect = mLastGhost;
					StClipRgnState clip;
					
					if (indicRect.left < frame.left)
						{
							if (indicRect.right > frame.left)
								indicRect.right = frame.left;
						}
					else
						{
							if (frame.right > indicRect.left)
								indicRect.left = frame.right;
						}
					clip.ClipToIntersection(indicRect);
					::RGBBackColor(&gAGAColorArray[2]);
					::EraseRect(&mLastGhost);
					DrawSlideTrack();
					GetIndicatorRect(indicRect);
					if (::SectRect(&indicRect, &mLastGhost, &overStep))
						DrawIndicator(true);
				}
			
			StColorPenState theState;
			theState.Normalize();
			mGhostPointer->CopyImage(GetMacPort(), frame);

			if ((!::EmptyRect(&overStep)) && (mGhostMask != nil))
				{
					StClipRgnState clip;
					::OffsetRgn(mGhostMask, -(*mGhostMask)->rgnBBox.left, -(*mGhostMask)->rgnBBox.top);
					::OffsetRgn(mGhostMask, frame.left, frame.top);
					RgnHandle theClipRegion = ::NewRgn();
					GetIndicatorRect(indicRect);
					::RectRgn(theClipRegion, &indicRect);
					::DiffRgn(theClipRegion, mGhostMask, theClipRegion);
					clip.ClipToIntersectionRgn(theClipRegion);
					DrawIndicator(true);
				}
			mLastGhost = frame;
			mLastPosition = inPoint;
		}
}

void LAGASquareHSlider::FinishTrackHotSpot (Point inPoint)

{
	Boolean hasColor = ::PaneInColor(this);
	Rect frame, indicRect;
	short newValue;

	CalcLocalFrameRect(frame);
	double size = (frame.right - frame.left) - 20;
	double division = size / (double)(mMaxValue - mMinValue);
			
	short MaxPt = frame.left + 10 + ((mMaxValue - mMinValue) * division);
	short MinPt = frame.left + 10;
	if (inPoint.h >= MaxPt)
		newValue = mMaxValue;
	else
		if (inPoint.h <= MinPt)
			newValue = mMinValue;
		else
			{
				newValue = ((inPoint.h - MinPt) / division) + mMinValue;
				short lowPos = frame.left + 10 + (newValue * division);
				short highPos = frame.left + 10 + ((newValue + 1) * division);
				if ((inPoint.h - lowPos) > (highPos - inPoint.h))
					newValue++;
			}
	
	if (newValue != mValue)
		{
			StClipRgnState clip;
			clip.Normalize();
			GetIndicatorRect(indicRect);
			if (hasColor)
				::RGBBackColor(&gAGAColorArray[2]);
			::EraseRect(&indicRect);
			::EraseRect(&mLastGhost);
			DrawSlideTrack();
			//	Update our control value and Broadcast it
			SetValue(newValue);
		}
	else
		{
			if (hasColor)
				::RGBBackColor(&gAGAColorArray[2]);
			::EraseRect(&mLastGhost);
			DrawSlideTrack();
		}
	//	We dont redraw the pointer here, because it will anyway be redrawn in the unpressed state
	//	in the method LAGASliderBase::HotSpotResult
}

