// ===========================================================================
//	LAGAGroupBox.h
// ===========================================================================
//	Apple Grayscale Appearance compliant primary and secondary group boxes
//	Copyright  1996 Chrisoft (Christophe ANDRES)  All rights reserved.
//
//	You may use this source code in any application (commercial, shareware, freeware,
//	postcardware, etc), but not remove this notice (no need to acknowledge the use of
//	this class in the about box)
//	You may not sell this source code in any form. This source code may be placed on 
//	publicly accessable archive sites and source code disks. It may not be placed on 
//	profit archive sites and source code disks without the permission of the author, 
//	Christophe ANDRES.
//	
//	    This source code is distributed in the hope that it will be useful,
//	    but WITHOUT ANY WARRANTY; without even the implied warranty of
//	    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
//
//	If you make any change or improvement on this class, please send the improved/changed
//	version to : chrisoft@calva.net or Christophe ANDRES
//                                     20, rue Prosper Mrime
//                                     67100 STRASBOURG
//                                     FRANCE
//
// ===========================================================================
//	LAGAGroupBox.cp			<- double-click + Command-D to see classes implementation
//
//	LAGAGroupBox is my implementation of the Apple Grayscale Appearance for System 7.5
//		primary group box, except that is can only have a text title and not a popup menu,
//		or a checkbox in its title.
//
//	LAGASecondaryGroupBox is my implementation of the Apple Grayscale Appearance for System 7.5
//		secondary group box, except that is can only have a text title and not a popup menu,
//		or a checkbox in its title.
//
//	Note : the CPPb declaration of LAGASecondaryGroupBox, as defined in LAGASeparator CPPb.rsrc
//					requires that the CPPb declaration of LAGAGroupBox is also present
//
//		This class requires AGAColors.cp to be present in your project
//		This class requires LGroupBox.cp to be present in your project
//
//		Version : 1.2
//
//		Change History (most recent first, date in US form : mm/dd/yy):
//
//						06/30/96	ca		Public release of version 1.2
//						06/28/96	ca		Added DisableSelf and EnableSelf methods
//						06/05/96	ca		Added RegisterClass method to ease registry
//														Added guard macros in header files
//														Increased version to 1.2
//						05/17/96	ca		Increased version to 1.1
//														Added copy constructor
//														Added "on the fly" constructor
//														Added change history
//						04/22/96	ca		class made available by Christophe ANDRES <chrisoft@calva.net>
//														(version 1.0)
//
//		To Do:
//

#ifndef _H_LAGAGroupBox
#define _H_LAGAGroupBox
#pragma once

#include <LGroupBox.h>

class LAGAGroupBox : public LGroupBox
{
	public		:
		enum { class_ID = 'AGA3' };
		static void RegisterClass ();																																							//	<06/05/96	ca>
		static LAGAGroupBox* CreateAGAGroupBoxStream (LStream* inStream);
		
		LAGAGroupBox (LStream *inStream);
		
		LAGAGroupBox (const LAGAGroupBox &inOriginal);																														//	<05/17/96	ca>
		
		LAGAGroupBox (const SPaneInfo &inPaneInfo, Str255 inString, ResIDT inTextTraitsID);												//	<05/17/96	ca>

	protected	:
		virtual void DrawText (const Rect &inRect);
		
		virtual void DrawBorder (const Rect &inRect);

		virtual void DisableSelf () { Refresh(); };																																//	<06/28/96	ca>

		virtual void EnableSelf () { Refresh(); };																																//	<06/28/96	ca>
};

class LAGASecondaryGroupBox : public LAGAGroupBox
{
	public		:
		enum { class_ID = 'AGA4' };
		static void RegisterClass ();																																							//	<06/05/96	ca>
		static LAGASecondaryGroupBox* CreateAGASecondaryGroupBoxStream (LStream* inStream);
		
		LAGASecondaryGroupBox (LStream *inStream);
		
		LAGASecondaryGroupBox (const LAGASecondaryGroupBox &inOriginal);																					//	<05/17/96	ca>
		
		LAGASecondaryGroupBox (const SPaneInfo &inPaneInfo, Str255 inString, ResIDT inTextTraitsID);							//	<05/17/96	ca>

	protected	:
		virtual void DrawBorder (const Rect &inRect);
};

#endif
